/*
 * drivers/video/geode/geodefb.h
 *   -- Geode framebuffer driver
 *
 * Copyright (C) 2005 Arcom Control Systems Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */
#ifndef __GEODEFB_H__
#define __GEODEFB_H__

#include <linux/fb.h>

#define GEODEFB_CRT_DEV   0x01
#define GEODEFB_PANEL_DEV 0x02

struct geodefb_panel_info {
  int refresh;
  int xres;
  int yres;
};


struct geodefb_mode {
  int xres;
  int yres;
  int refresh;
  int depth;
};

#define FB_HZ     0
#define FB_HTOTAL 1
#define FB_VTOTAL 2
#define FB_FREQ   3

#define GEODEFB_GET_HZ(var)       geodefb_get_timings(var, 0, FB_HZ)
#define GEODEFB_GET_HTOTAL(var)   geodefb_get_timings(var, 0, FB_HTOTAL)
#define GEODEFB_GET_VTOTAL(var)   geodefb_get_timings(var, 0, FB_VTOTAL)
#define GEODEFB_GET_FREQ(var, hz) geodefb_get_timings(var, hz, FB_FREQ)

unsigned int geodefb_get_devices(void);
int geodefb_get_panel_info(struct geodefb_panel_info *);
void geodefb_parse_mode(const char *, struct geodefb_mode *);
int geodefb_get_timings(struct fb_var_screeninfo *var, int hz, int flag);

struct geodefb_info;

struct geode_dc_ops {
	void (*set_mode)(struct fb_info *);
	void (*set_palette_reg)(struct fb_info *, unsigned, unsigned, unsigned, unsigned);
};

struct geode_vid_ops {
	void (*set_dclk)(struct fb_info *);
	void (*configure_display)(struct fb_info *);
	int  (*blank_display)(struct fb_info *, int blank_mode);
};

struct geodefb_par {
	int enable_crt;
	int panel_x; /* dimensions of an attached flat panel, non-zero => enable panel */
	int panel_y;
	void __iomem *dc_regs;
	void __iomem *vid_regs;
	struct geode_dc_ops  *dc_ops;
	struct geode_vid_ops *vid_ops;
};

#endif /* !__GEODEFB_H__ */
