/* <LIC_AMD_STD>
 * Copyright (c) 2004-2005 Advanced Micro Devices, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * The full GNU General Public License is included in this distribution in the
 * file called COPYING
 * </LIC_AMD_STD>  */
/* <CTL_AMD_STD>
 * </CTL_AMD_STD>  */
/* <DOC_AMD_STD>
 * Geode hardware specific routines
 * </DOC_AMD_STD>  */

#include "geodefb.h"

/* Device specific stuff */
/* Each platform should define READ_VG */

#ifdef CONFIG_FB_GEODE_LX

#define READ_VG(reg) lx_read_vg(reg)

extern void cim_outw(unsigned short port, unsigned short data);
extern unsigned short cim_inw(unsigned short port);

unsigned short lx_read_vg(unsigned char reg) {

  unsigned short out;

  cim_outw(0xAC1C, 0xFC53);
  cim_outw(0xAC1C, 0x0200 | reg);

  out = cim_inw(0xAC1E);
  
  return out;
}

#endif

#ifndef READ_VG
#error "geodefb:  No processor was selected - probably due to a corrupted config"
#endif

/* List of refresh values from the FP config register */

static int geodefb_panel_refresh[7] = 
  { 60, 70, 72, 75, 85, 90, 100 };

/* List of panel resolutiosn from the FP config register */

static struct {
  int xres;
  int yres;
} geodefb_panel_res[] = {
  { 320, 240 },
  { 640, 480 },
  { 800, 600 },
  { 1024, 768 },
  { 1152, 864 },
  { 1280, 1024 },
  { 1600, 1200 }
};

int geodefb_get_panel_info(struct geodefb_panel_info *info) {
  
  unsigned short reg = READ_VG(0x02);
  
  /* refresh rate */
  
  info->refresh = geodefb_panel_refresh[(reg >> 13) & 0x7];
  info->xres = geodefb_panel_res[(reg >> 3) & 0x07].xres;
  info->yres = geodefb_panel_res[(reg >> 3) & 0x07].yres;

  return 0;
}

unsigned int geodefb_get_devices(void) {
  unsigned short reg = READ_VG(0x0);

  switch((reg >> 8) & 0x7) {
  case 0:
    return GEODEFB_CRT_DEV;

  case 1:
    return GEODEFB_PANEL_DEV;
    
  case 5:
    return GEODEFB_CRT_DEV | GEODEFB_PANEL_DEV;
  }

  return 0;
}
