/* <LIC_AMD_STD>
 * Copyright (c) 2004-2005 Advanced Micro Devices, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * The full GNU General Public License is included in this distribution in the
 * file called COPYING
 * </LIC_AMD_STD>  */
/* <CTL_AMD_STD>
 * </CTL_AMD_STD>  */
/* <DOC_AMD_STD>
 * Geode LX framebuffer driver
 * </DOC_AMD_STD>  */

/* #include <linux/config.h> */
#include <linux/module.h>
#include <linux/kernel.h>

#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/pm.h>
#include <linux/platform_device.h>

/* For the console notifier */
/* #include <linux/vt_kern.h> */

#include <cim_mem.h>

#include <cim/cim_parm.h>
#include <cim/cim_rtns.h>

#include "geodefb.h"
#include "geodefb_lx.h"

#include "build_num.h"

extern char vmode[15];
extern int compression;
extern int vfreq;
extern int maxmem;
extern int accel;

#ifdef CONFIG_FB_GEODE_PANEL
extern int flatpanel;
extern int fphwscale;
static struct geodefb_panel_info panel_info;
#endif

/* Cimarron */

extern void cim_outb(unsigned short port, unsigned char data);
extern unsigned char cim_inb(unsigned short port);
extern void cim_outw(unsigned short port, unsigned short data);

extern struct notifier_block *console_notifier_list;

/* Macros */

#define LXFB_FBINFO_FLAGS (FBINFO_DEFAULT | \
			   FBINFO_HWACCEL_COPYAREA | \
			   FBINFO_HWACCEL_FILLRECT | \
			   FBINFO_HWACCEL_IMAGEBLIT | \
			   FBINFO_HWACCEL_XPAN | \
			   FBINFO_HWACCEL_YPAN)

#define GEODEFB_DEFAULT_XRES  640
#define GEODEFB_DEFAULT_YRES  480
#define GEODEFB_DEFAULT_DEPTH  16

/* Local variables */
static struct fb_info *fbinfo;
static int lxfb_palette[16];

static int  active_devices;

static struct geodefb_mode initial_mode = {
	xres:  GEODEFB_DEFAULT_XRES,
	yres:  GEODEFB_DEFAULT_YRES,
	depth: GEODEFB_DEFAULT_DEPTH
};
static struct geodefb_mode initial_mode;

struct fb_fix_screeninfo lxfb_fix = {
        id:        "AMD LX",
	type:      FB_TYPE_PACKED_PIXELS,
	visual:    FB_VISUAL_TRUECOLOR,
	xpanstep:  8,
	ywrapstep: 1,
	ypanstep:  0,
	accel:     FB_ACCEL_NONE
};

/* ======= Utility variables ======= */

int
lxfb_get_pitch(int xres, int bpp)
{
	int d = xres * (bpp >> 3);
   
	if (d > 4096) return 8192;
	if (d > 2048) return 4096;
	if (d > 1024) return 2048;

	return 1024;
}

static int
lxfb_get_refresh(VG_DISPLAY_MODE *mode) {
	switch(mode->internal_flags & VG_SUPPORTFLAG_HZMASK) {
		case VG_SUPPORTFLAG_56HZ: return 56;
		case VG_SUPPORTFLAG_60HZ: return 60;
		case VG_SUPPORTFLAG_70HZ: return 70;
		case VG_SUPPORTFLAG_72HZ: return 72;
		case VG_SUPPORTFLAG_75HZ: return 75;
		case VG_SUPPORTFLAG_85HZ: return 85;
		case VG_SUPPORTFLAG_90HZ: return 90;
		case VG_SUPPORTFLAG_100HZ: return 100;
	}
	return 60;
}

void
lxfb_clear_active(struct lxfb_par *par, int xres, int yres, int bpp) 
{
	int pitch = lxfb_get_pitch(xres,bpp);

#ifdef CONFIG_FB_GEODE_ACCEL
	if (par->accel == ACCEL_ACTIVE) {
		gp_declare_blt(0);
		gp_set_raster_operation(0xF0);  
		gp_set_solid_pattern(0);
		gp_set_strides(pitch, 0);
    
		gp_pattern_fill(0, xres, yres);
		gp_wait_blt_pending();    
	} else 
#endif
	{
		memset(par->fbvirt, 0, yres * pitch);
	}
}

/* From 2.4 - check to see if the timings have changed */
/* AUDIT:  Is this still needed?                       */

static int lxfb_timings_changed(struct fb_var_screeninfo *old, 
				struct fb_var_screeninfo *new) {
	
	if (old->pixclock != new->pixclock) return 1;
	if (old->left_margin != new->left_margin) return 1;
	if (old->right_margin != new->right_margin) return 1;
	if (old->upper_margin != new->upper_margin) return 1;
	if (old->lower_margin != new->lower_margin) return 1;
	if (old->hsync_len != new->hsync_len) return 1;
	if (old->vsync_len != new->vsync_len) return 1;

	return 0;
}


/* Not just timings - this should result in a fairly sane var structure */

static void
lxfb_set_timings(struct fb_var_screeninfo *var, int mnum) {

	VG_DISPLAY_MODE mode;
	vg_get_display_mode_information(mnum, &mode);
  
	var->hsync_len = mode.hsyncend - mode.hsyncstart;
	var->vsync_len = mode.vsyncend - mode.vsyncstart;
  
	var->right_margin = var->left_margin =
		(mode.htotal - mode.hactive - var->hsync_len) / 2;
	var->upper_margin = var->lower_margin =
		(mode.vtotal - mode.vactive - var->vsync_len) / 2;

	var->pixclock = GEODEFB_GET_FREQ(var, lxfb_get_refresh(&mode));

	var->sync = 0;
  
	if (!(mode.flags & VG_MODEFLAG_NEG_HSYNC))
		var->sync |= FB_SYNC_HOR_HIGH_ACT;
  
	if (!(mode.flags & VG_MODEFLAG_NEG_VSYNC))
		var->sync |= FB_SYNC_VERT_HIGH_ACT;
  
	if (mode.flags & VG_MODEFLAG_INTERLACED) {
		var->vmode = FB_VMODE_INTERLACED;
  
		if (mode.flags & VG_MODEFLAG_INT_LINEDOUBLE)
			var->vmode |= FB_VMODE_DOUBLE;
	}
	else
		var->vmode = FB_VMODE_NONINTERLACED;

	var->width = var->height = -1;  

	switch (var->bits_per_pixel) {
    
		case 8: 
			var->red.offset = 0;
			var->red.length = 6;
			var->red.msb_right = 0;
			var->blue = var->green = var->red;   
			break;
    
		case 16: /* 5-6-5 */
			var->red.offset = 11;
			var->red.length = 5;
			var->green.offset = 5;
			var->green.length = 6;
			var->blue.offset = 0;
			var->blue.length = 5;
			var->transp.offset = 0;
			var->transp.length = 0;
			break;

		case 32: /* A-8-8-8 */
			var->transp.offset = 24;
			var->transp.length = 8;
			var->red.offset = 16;
			var->red.length = 8;
			var->green.offset = 8;
			var->green.length = 8;
			var->blue.offset = 0;
			var->blue.length = 8;   
			break;
	}

	if (var->xres > var->xres_virtual) 
		var->xres_virtual = var->xres;
  
	if (var->yres > var->yres_virtual)
		var->yres_virtual = var->yres;
  
	if (var->xoffset < 0) var->xoffset = 0;
	if (var->yoffset < 0) var->yoffset = 0;

	if (var->xoffset > var->xres_virtual - var->xres) 
		var->xoffset = var->xres_virtual - var->xres;
  
	if (var->yoffset > var->yres_virtual - var->yres) 
		var->yoffset = var->yres_virtual - var->yres;
  
	var->red.msb_right = 0;
	var->green.msb_right = 0;
	var->blue.msb_right = 0;
	var->transp.msb_right = 0;
}

static void
lxfb_set_mode(struct lxfb_par *par, struct fb_var_screeninfo *var) {

	int bpp = var->bits_per_pixel;
	int hz = GEODEFB_GET_HZ(var);  

	if (compression)
		vg_set_compression_enable(0);
  
#ifdef CONFIG_FB_GEODE_PANEL
	if (active_devices & GEODEFB_PANEL_DEV) {
		int axres = var->xres, ayres = var->yres;
		int flags = (active_devices & GEODEFB_CRT_DEV) ? VG_MODEFLAG_CRT_AND_FP : 0;
    
		if (fphwscale && panel_info.xres >= var->xres && var->xres <= 1024) {
			axres = panel_info.xres;
			ayres = panel_info.yres;
		}
    
		vg_set_panel_mode(var->xres, var->yres, axres, ayres, 
				  panel_info.xres, panel_info.yres, bpp, flags);
    
		vg_set_border_color(0x0);  /* Make sure the border is clear */
	}
	else if (active_devices & GEODEFB_CRT_DEV)
#endif
	{
		vg_set_display_mode(var->xres, var->yres,
				    var->xres, var->yres, bpp, hz, 0);
    
	}
  
	if (var->xres_virtual > var->xres) 
		vg_set_display_pitch(lxfb_get_pitch(var->xres_virtual, bpp));

	df_set_crt_enable(DF_CRT_ENABLE);

#ifdef CONFIG_FB_GEODE_ACCEL
	gp_set_bpp(bpp);
#endif

	lxfb_clear_active(par, var->xres_virtual, var->yres_virtual, bpp);  

	/* Try to turn on compression if we can */
	/* We do not enable compresssion if panning is even possible */
  
	if (compression && var->xres_virtual == var->xres &&
	    var->yres_virtual == var->yres) {
		VG_COMPRESSION_DATA data;
    
		data.pitch = 544;
		data.size = 544;
		data.compression_offset = var->yres * lxfb_get_pitch(var->xres, bpp);
		vg_configure_compression(&data);

		vg_set_compression_enable(1);
	}

	par->vfreq = hz;
};

static int
lxfb_find_mode(struct lxfb_par *par, struct fb_var_screeninfo *var, 
	       struct geodefb_mode *mode) {

	VG_QUERY_MODE query;
	int mnum = 0;

	query.query_flags = VG_QUERYFLAG_BPP | VG_QUERYFLAG_REFRESH;
	query.bpp = mode->depth;
  
#ifdef CONFIG_FB_GEODE_PANEL
	if (active_devices & GEODEFB_PANEL_DEV) {
		query.query_flags |= VG_QUERYFLAG_PANEL;
    		query.query_flags  |= VG_QUERYFLAG_PANELWIDTH | VG_QUERYFLAG_PANELHEIGHT;
		query.panel_width = panel_info.xres;
		query.panel_height = panel_info.yres;
		
		query.hz = panel_info.refresh;
	} else
#endif
	{
		query.query_flags |= VG_QUERYFLAG_ACTIVEWIDTH | VG_QUERYFLAG_ACTIVEHEIGHT;

		query.active_width = mode->xres;
		query.active_height = mode->yres;
		query.hz = mode->refresh;
	}

	mnum = vg_get_display_mode_index(&query);
	if (mnum < 0) return -EINVAL;
  
	var->xres_virtual = var->xres = mode->xres;
	var->yres_virtual = var->yres = mode->yres;

	if (mode->depth <= 8) 
		var->bits_per_pixel = 8;
	else if (mode->depth <= 16)
		var->bits_per_pixel = 16;
	else if (mode->depth <= 32)
		var->bits_per_pixel = 32;
  
	lxfb_set_timings(var, mnum);

	if (par->accel & ACCEL_AVAILABLE_MASK)
		var->accel_flags = 1;

	return 0;
}

/* ======= FB Functions ======= */

static int
lxfb_check_var(struct fb_var_screeninfo *var, struct fb_info *info) {

	struct lxfb_par *par = (struct lxfb_par *) info->par;
	VG_QUERY_MODE query;
	int mode;
	unsigned int mem;

	/* Adjust the bpp if it comes in screwy */

	if (var->bits_per_pixel <= 8)
		var->bits_per_pixel = 8;
	else if (var->bits_per_pixel <= 16)
		var->bits_per_pixel = 16;
	else if (var->bits_per_pixel <= 32)
		var->bits_per_pixel = 32;

	/* See if we have enough room for this mode */

	mem = lxfb_get_pitch(var->xres, var->bits_per_pixel) * var->yres;
  
	if (mem > par->fbsize) 
		return -EINVAL;

	query.query_flags = VG_QUERYFLAG_BPP | VG_QUERYFLAG_REFRESH;
	query.bpp = var->bits_per_pixel;
		
#ifdef CONFIG_FB_GEODE_PANEL
	if (active_devices & GEODEFB_PANEL_DEV) {
		query.query_flags |= VG_QUERYFLAG_PANEL;
    		query.query_flags  |= VG_QUERYFLAG_PANELWIDTH | VG_QUERYFLAG_PANELHEIGHT;
		query.panel_width = panel_info.xres;
		query.panel_height = panel_info.yres;
		query.hz = panel_info.refresh;   
	}
	else if (active_devices & GEODEFB_CRT_DEV)
#endif
	{
		query.query_flags |= VG_QUERYFLAG_ACTIVEWIDTH | VG_QUERYFLAG_ACTIVEHEIGHT;

		query.active_width = var->xres;
		query.active_height = var->yres;

		if (lxfb_timings_changed(&info->var, var))
			query.hz = GEODEFB_GET_HZ(var);
		else
			query.hz = par->vfreq;
	}

	mode = vg_get_display_mode_index(&query);

	if (mode < 0)
		return -EINVAL;

	/* Set up the timing tables correctly for the given mode */
	lxfb_set_timings(var, mode);

	/* Don't allow acceleration, even if the user asks for it */

	if (!(par->accel & ACCEL_AVAILABLE_MASK))
		var->accel_flags = 0;
		
	return 0;
}

static int
lxfb_set_par(struct fb_info *info) {

	struct lxfb_par *par = (struct lxfb_par *) info->par;
	
	if (par->accel & ACCEL_ACTIVE_MASK)
		par->accel = ACCEL_NOT_READY;
	
	/* Set up the pointers before we try to set the mode */

#ifdef CONFIG_FB_GEODE_ACCEL
	if (info->var.accel_flags && (par->accel & ACCEL_AVAILABLE_MASK)) {
		gp_set_frame_buffer_base (par->fbphys, par->fbsize);
		gp_set_command_buffer_base(par->gpphys, 0, par->gpsize);
	}
#endif

	lxfb_set_mode(par, &info->var);
 		
	info->fix.line_length = lxfb_get_pitch(info->var.xres_virtual, 
					       info->var.bits_per_pixel);
	info->fix.visual = (info->var.bits_per_pixel == 8) 
		? FB_VISUAL_PSEUDOCOLOR : FB_VISUAL_TRUECOLOR;
  
	info->cmap.len = (info->var.bits_per_pixel == 8) ? 256 : 16;
      
	if (info->var.accel_flags) {
		if (par->accel == ACCEL_NOT_READY || par->accel == ACCEL_USER_DISABLED)
			par->accel = ACCEL_ACTIVE;
	}
	else {
		if (par->accel == ACCEL_NOT_READY)
			par->accel = ACCEL_USER_DISABLED;
	}
	
	return 0;
}

static int
lxfb_setcolreg(unsigned regno, unsigned red, unsigned green,
	       unsigned blue, unsigned transp, struct fb_info *info) {

	if ((info->var.bits_per_pixel == 8 && regno > 255) || regno > 15)
		return 1;
  
	switch(info->var.bits_per_pixel) {
		case 8: {
			u32 color = ((red & 0xFF00) << 8) | (green & 0xFF00) | ((blue & 0xFF00) >> 8);
			vg_set_display_palette_entry(regno, color);
			vg_set_display_palette_entry(regno ^ 0xFF, color ^ 0xFFFFFF);
		}
			break;

		case 16:
			((u32*)info->pseudo_palette)[regno] = (red & 0xF800) |
				((green & 0xFC00) >> 5) | ((blue & 0xF800) >> 11);
			break;

		case 32:
			((u32*)info->pseudo_palette)[regno] =
				((transp & 0xFF00) <<16)        |
				((red & 0xFF00) << 8)           |
				((green & 0xFF00))              |
				((blue & 0xFF00)>>8);
			break;
	}
  
	return 0;
}

static int
lxfb_pan_display(struct fb_var_screeninfo *var, struct fb_info *info) {
  
	u32 offset = 0;
  
	if (var->xoffset < 0 || var->xres + var->xoffset > var->xres_virtual)
		return -EINVAL;
     
	if (var->xoffset < 0 || var->yres + var->yoffset > var->yres_virtual)
		return -EINVAL;

	/* Calculate the fb offset */

	offset = (var->yoffset * vg_get_display_pitch()) + 
		(var->xoffset * (var->bits_per_pixel >> 3));
     
	vg_set_display_offset(offset);  
	return 0;
}

static int
lxfb_blank(int blank_mode, struct fb_info *info) {

#ifdef CONFIG_FB_GEODE_PANEL
	/* Blank the panel if we have one attached */

	if (active_devices & GEODEFB_PANEL_DEV)
		df_set_panel_enable( (blank_mode) ? 0 : 1);
#endif
  
	switch(blank_mode) {
		case VESA_POWERDOWN:
			df_set_crt_enable(DF_CRT_DISABLE);
			break;

		case VESA_VSYNC_SUSPEND:
			df_set_crt_enable(DF_CRT_SUSPEND);
			break;
    
		case VESA_HSYNC_SUSPEND:
			df_set_crt_enable(DF_CRT_STANDBY);
			break;
    
		default:
			df_set_crt_enable(DF_CRT_ENABLE);
			break;
	}

	return 0;
}


#ifdef CONFIG_FB_GEODE_ACCEL

int lxfb_sync(struct fb_info *info) {
	struct lxfb_par *par = (struct lxfb_par *) info->par;
  
	if (par->accel == ACCEL_ACTIVE)
		gp_wait_blt_pending();

	return 0;
}
#endif

#ifdef CONFIG_FB_GEODE_ACCEL
void lxfb_imageblit(struct fb_info *, const struct fb_image *);
void lxfb_copyarea(struct fb_info *, const struct fb_copyarea *);
void lxfb_fillrect(struct fb_info *, const struct fb_fillrect *);
#endif

static struct fb_ops lxfb_fb_ops = {
	owner:          THIS_MODULE,

	fb_setcolreg:   lxfb_setcolreg,
	fb_check_var:   lxfb_check_var,
	fb_set_par:     lxfb_set_par, 
	fb_pan_display: lxfb_pan_display,
	fb_blank:       lxfb_blank,

#ifdef CONFIG_FB_GEODE_ACCEL
	fb_fillrect:    lxfb_fillrect,
	fb_copyarea:    lxfb_copyarea,
	fb_imageblit:   lxfb_imageblit,
	fb_sync:        lxfb_sync,
#else
	fb_fillrect:    cfb_fillrect,
	fb_copyarea:    cfb_copyarea,
	fb_imageblit:   cfb_imageblit,
#endif

/* 	fb_cursor:      soft_cursor, */
};
      
static int 
lxfb_mem_init(struct lxfb_par *par) 
{  
	int ret;
  
	par->fbsize = (maxmem * 1024);
	if (par->fbsize > cim_get_fb_active())
		par->fbsize = cim_get_fb_active();

	KWRN("Allocating %dK of framebuffer memory\n", par->fbsize / 1024);

	par->fbphys = cim_get_fb_base();
	par->fbvirt = cim_get_fb_ptr();

	ret = cim_get_memory("geodefb", "framebuffer", par->fbsize, CIM_F_PUBLIC);
	if (!ret) return -ENOMEM;

	/* Set up acceleration - if this fails, its not fatal */

#ifdef CONFIG_FB_GEODE_ACCEL
	if (accel) {
		par->gpphys = cim_get_cmd_base();
		par->gpsize = cim_get_cmd_size();
    
		if (par->gpphys) 
			par->accel = ACCEL_NOT_READY;
	}
#endif
  
	return 0;
}

static GP_SAVE_RESTORE gpsave;
static VG_SAVE_RESTORE vgsave;
static DF_SAVE_RESTORE dfsave;

int lxfb_suspend(struct device *dev, u32 state, u32 level) {
      	
/* 	if (level == SUSPEND_POWER_DOWN) { */
/* #ifdef CONFIG_FB_GEODE_PANEL */
/* 		if (active_devices & GEODEFB_PANEL_DEV) */
/* 			df_set_panel_enable(0); */
/* #endif */
/* 		df_set_crt_enable(DF_CRT_DISABLE); */
		
/* 		gp_save_state(&gpsave); */
/* 		vg_save_state(&vgsave); */
/* 		df_save_state(&dfsave); */
/* 	} */

	return 0;
}

int lxfb_resume(struct device *dev, u32 level) {
      	
/* 	if (level == RESUME_ENABLE) { */
/* 		vg_restore_state(&vgsave); */
/* 		gp_restore_state(&gpsave); */
/* 		df_restore_state(&dfsave); */
		
/* 		df_set_crt_enable(DF_CRT_ENABLE); */
		
/* #ifdef CONFIG_FB_GEODE_PANEL */
/* 		if (active_devices & GEODEFB_PANEL_DEV) */
/* 			df_set_panel_enable(1); */
/* #endif */
/* 	} */

	return 0;
}				

/* static int  */
/* lxfb_console_notify(struct notifier_block *self, */
/* 		    unsigned long action, void *data) { */

/* 	struct lxfb_par *par = (struct lxfb_par *) fbinfo->par; */

/* 	if (action == CONSOLE_EVENT_SWITCH_TEXT) { */

/* 		if (par->accel == ACCEL_KERNEL_DISABLED) { */
/* 			gp_set_frame_buffer_base (par->fbphys, par->fbsize); */
/* 			gp_set_command_buffer_base(par->gpphys, 0, par->gpsize); */
      
/* 			par->accel = ACCEL_ACTIVE; */
/* 		} */
/* 	} */
/* 	else if (action == CONSOLE_EVENT_SWITCH_GRAPHICS) { */
/* 		if (par->accel == ACCEL_ACTIVE) { */
/* 			gp_wait_blt_pending(); */
/* 			par->accel = ACCEL_KERNEL_DISABLED; */
/* 		} */
/* 	} */

/* 	return 0; */
/* } */

/* static struct notifier_block lxfb_console_notifier = { */
/*         .notifier_call  = lxfb_console_notify, */
/* }; */

int __init
lxfb_init(struct device *device)
{  
	int ret, parsed = 0;
	struct platform_device *dev = to_platform_device(device);

	fbinfo = framebuffer_alloc(sizeof(struct lxfb_par), &dev->dev);

	if (!fbinfo) {
		KERR("Unable to allocatethe framebuffer memory.\n");
		return -ENOMEM;
	}

	active_devices = geodefb_get_devices();

	initial_mode.refresh = vfreq;
	
	if (vmode[0] != 0) {
		geodefb_parse_mode(vmode, &initial_mode);
		parsed = 1;
	}

#ifdef CONFIG_FB_GEODE_PANEL
	if (active_devices & GEODEFB_PANEL_DEV) {

		if (flatpanel) {
			geodefb_get_panel_info(&panel_info);
			if (!parsed) {
				initial_mode.xres = panel_info.xres;
				initial_mode.yres = panel_info.yres;
			}
			initial_mode.refresh = panel_info.refresh;
			KWRN("TFT support enabled.");
		}	  
		else
			active_devices &= ~GEODEFB_PANEL_DEV;
	}
#endif

	if (lxfb_mem_init((struct lxfb_par *) fbinfo->par)) {
		KERR("Unable to initialize the graphics engine.\n");
		kfree(fbinfo);
		return -ENODEV;
	}

	fbinfo->screen_base = ((struct lxfb_par *) fbinfo->par)->fbvirt;
	fbinfo->fbops = &lxfb_fb_ops;
	fbinfo->fix = lxfb_fix;
	fbinfo->pseudo_palette = lxfb_palette;
 
	/* Since we have a nifty screen to screen blit, we use that for
	   scrolling - look ma, no bus!
	*/

	fbinfo->flags = LXFB_FBINFO_FLAGS | FBINFO_READS_FAST;  

	ret = lxfb_find_mode(fbinfo->par, &fbinfo->var, &initial_mode);

	if (ret < 0) {
		KERR("Invalid mode %dx%d@%d-%d\n", initial_mode.xres, initial_mode.yres,
		     initial_mode.refresh, initial_mode.depth);

		return -EINVAL;
	}

	KWRN("Initial mode: %dx%d@%d-%d\n", 
	     initial_mode.xres, initial_mode.yres,
	     initial_mode.refresh, initial_mode.depth);
		     
	fbinfo->var.activate |= FB_ACTIVATE_NOW;

	fb_alloc_cmap(&fbinfo->cmap, 256, 0);

	/* ==== register ==== */

/* 	notifier_chain_register(&console_notifier_list, */
/* 				&lxfb_console_notifier); */

	if (register_framebuffer(fbinfo) < 0) {
		framebuffer_release(fbinfo);
		KERR("Unable to register the framebuffer\n");
		return -EINVAL;
	}

	printk(KERN_INFO "%s\n", AMD_VERSION);
	return 0;
}

void __exit 
lxfb_cleanup(void) {
/* 	notifier_chain_unregister(&console_notifier_list,  */
/* 				  &lxfb_console_notifier); */

	unregister_framebuffer((struct fb_info *) fbinfo);
	framebuffer_release(fbinfo);
}


