/* <LIC_AMD_STD>
 * Copyright (c) 2004-2005 Advanced Micro Devices, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * The full GNU General Public License is included in this distribution in the
 * file called COPYING
 * </LIC_AMD_STD>  */
/* <CTL_AMD_STD>
 * </CTL_AMD_STD>  */
/* <DOC_AMD_STD>
 * Geode LX acceleration routines
 * </DOC_AMD_STD>  */

/* #include <linux/config.h> */
#include <linux/module.h>
#include <linux/kernel.h>

#include <cim/cim_rtns.h>

#include "geodefb.h"
#include "geodefb_lx.h"

extern int lxfb_get_pitch(int, int);

/* Matches ROP_ defines in fb.h */
static unsigned char lx_rops[] = { 0xF0, 0x55 };

void 
lxfb_imageblit(struct fb_info *info, const struct fb_image *image) {

  struct lxfb_par *par = (struct lxfb_par *) info->par;  
  int pitch = lxfb_get_pitch(info->var.xres_virtual, info->var.bits_per_pixel);
  u32 dst = (image->dy * pitch) + (image->dx * (info->var.bits_per_pixel >> 3));

  if (par->accel != ACCEL_ACTIVE) {
    cfb_imageblit(info, image);
    return;
  }

  if (image->depth == 1) {
    u32 fg = 0, bg = 0;
       
    switch(info->var.bits_per_pixel) {
    case 8: 
      bg = image->bg_color;
      fg = image->fg_color;      
      break;
      
    case 16:
    case 32:
      bg = ((u32 *) info->pseudo_palette)[image->bg_color];
      fg = ((u32 *) info->pseudo_palette)[image->fg_color];
      break;
    }
    
    gp_declare_blt(0);
    gp_set_raster_operation(0xCC);
    gp_set_strides(pitch, 0);    
    gp_set_mono_source(bg, fg, 0);
    
    gp_mono_bitmap_to_screen_blt(dst, 0,
				 image->width, image->height, 
				 (unsigned char *) image->data, 
				 image->width / 8);
  } else if (image->depth == info->var.bits_per_pixel) {

    gp_declare_blt(0);
    gp_set_raster_operation(0xCC);
    gp_set_strides(pitch, 0);    

    gp_color_bitmap_to_screen_blt(dst, 0, image->width, image->height,
				  (unsigned char *) image->data,
				  image->width * (image->depth >> 3));    
  }
  else {
    int flags = 0;
          
    if (image->depth == 8) {
      u32 colors[256];
      memcpy(colors, (u32 *) info->pseudo_palette, 256 * sizeof(u32));
      
      gp_program_lut((unsigned long *) colors, 1);
      flags = CIMGP_BLTFLAGS_PRES_LUT;
    }
    
    gp_declare_blt(flags);
    gp_set_raster_operation(0xCC);
    gp_set_strides(pitch, 0);    

    switch(image->depth) {
    case 8:
      gp_set_source_format(CIMGP_SOURCE_FMT_8BPP_INDEXED);
      break;
    case 16:
      gp_set_source_format(CIMGP_SOURCE_FMT_0_5_6_5);
      break;
    case 32:
      gp_set_source_format(CIMGP_SOURCE_FMT_8_8_8_8);
      break;
    }

    gp_color_convert_blt(dst, 0, image->width, image->height,
			 (unsigned char *) image->data,
			 image->width * (image->depth >> 3));
  }
}

void 
lxfb_copyarea(struct fb_info *info, const struct fb_copyarea *area) {
  
  struct lxfb_par *par = (struct lxfb_par *) info->par;
  int pitch = lxfb_get_pitch(info->var.xres_virtual, info->var.bits_per_pixel);
  int flags = 0;

  u32 src = (area->sy * pitch) + (area->sx * (info->var.bits_per_pixel >> 3));
  u32 dst = (area->dy * pitch) + (area->dx * (info->var.bits_per_pixel >> 3));

  if (par->accel != ACCEL_ACTIVE) {
    cfb_copyarea(info, area);
    return;
  }
  
  gp_declare_blt(0);
  gp_set_raster_operation(0xCC);
  gp_set_strides(pitch, pitch);  
  gp_set_solid_pattern(0);
  
  if (area->dx > area->sx) flags |= CIMGP_NEGXDIR;
  if (area->dy > area->sy) flags |= CIMGP_NEGYDIR;

  gp_screen_to_screen_blt(dst, src, area->width, area->height, flags);    
}
  
void 
lxfb_fillrect(struct fb_info *info, const struct fb_fillrect *rect) {

  struct lxfb_par *par = (struct lxfb_par *) info->par;  
  int pitch = lxfb_get_pitch(info->var.xres_virtual, info->var.bits_per_pixel);
  u32 dst = (rect->dy * pitch) + (rect->dx * (info->var.bits_per_pixel >> 3));

  if (par->accel != ACCEL_ACTIVE) {
    cfb_fillrect(info, rect);
    return;
  }

  gp_declare_blt(0);
  gp_set_raster_operation(lx_rops[rect->rop]);  
  
  if (lx_rops[rect->rop] != 0x55)
    gp_set_solid_pattern(rect->color);

  gp_set_strides(pitch, 0);
    
  gp_pattern_fill(dst, rect->width, rect->height);
}
