/* <LIC_AMD_STD>
 * Copyright (c) 2004-2005 Advanced Micro Devices, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * The full GNU General Public License is included in this distribution in the
 * file called COPYING
 * </LIC_AMD_STD>  */
/* <CTL_AMD_STD>
 * </CTL_AMD_STD>  */
/* <DOC_AMD_STD>
 * Geode framebuffer header file
 * </DOC_AMD_STD>  */

#include <linux/string.h>
#include "geodefb.h"

void geodefb_parse_mode(const char *string, struct geodefb_mode *mode) {
  
  char *next, *ptr = (char *) string;
  int values[4] = { -1, -1, -1, -1 };
  int flag = 0;

  while(*ptr) {
    switch(*ptr) {
    case 'x':
      flag = 1;
      ptr++;
      break;
    case '@':
      flag = 2;
      ptr++;
      break;
    case '-':
      flag = 3;
      ptr++;
      break;
    case '0'...'9':
      values[flag] = simple_strtoul(ptr, &next, 0);
      ptr = next;
      break;
    }
  }
  
  if (values[0] != -1) mode->xres = values[0];
  if (values[1] != -1) mode->yres = values[1];
  if (values[2] != -1) mode->refresh = values[2];
  if (values[3] != -1) mode->depth = values[3];
}

/* from fb.modes:
   P = 1E12 / (V*F*H)
   F = 1E12 / (V*P*H)

   where P=pixclock, F=frequency, V=vtotal, H=htotal
*/

/* We can only use 32 bit integers, so we split 1E12 into something
   more manageable */

static inline int VAR2FREQ(int htotal, int vtotal, int refresh) {
	   unsigned long h;

	   h = 1953125000 / htotal;
	   h = h * 512 / refresh;
	   return (int) (h / vtotal);
}

static inline int VAR2HZ(int htotal, int vtotal, int pixclock) {
	unsigned long v;
	
        v = 488281250 / htotal;
	v = v * 2048 / vtotal;
	return (int) ((((v * 100) / pixclock) + 50) / 100);
}

static int HTOTAL(struct fb_var_screeninfo *var) {
  return (var->left_margin + var->xres + var->right_margin + var->hsync_len);
}

static int VTOTAL(struct fb_var_screeninfo *var) {
  return (var->upper_margin + var->yres + var->lower_margin + var->vsync_len);
}

int geodefb_get_timings(struct fb_var_screeninfo *var, int hz, int flag) {
  
  unsigned int htotal = HTOTAL(var);
  unsigned int vtotal = VTOTAL(var);

  switch(flag) {
  case FB_HTOTAL: return htotal;
  case FB_VTOTAL: return vtotal;
  case FB_HZ:  return VAR2HZ(htotal, vtotal, var->pixclock);
  case FB_FREQ: return VAR2FREQ(htotal, vtotal, hz);
  }

  return -1;
}
